<?php

class Database{
	private $connection;
	public $query;
	private $num_rows;
	private $where;
	private $dbOptions = array();

	/* CONNECT */
	public function connect($server, $user, $pass, $dbname, $port = null){
		$this->connection = new mysqli($server, $user, $pass, $dbname, $port);
		if($this->connection->connect_error){
			mysqli_set_charset($this->connection, "utf8");
			exit ("Failed to connect to MySQL: " . $this->connection->connect_error);
		}else{
			mysqli_set_charset($this->connection, "utf8");
		}

		$this->dbOptions = array(			
			'database' => $dbname,
		);

		return $this->connection;
	}

	/**
	 * QUERY
	 *
	 * @param $query_param
	 *
	 * @return array
	 */
	public function query($query_param, $escape = false){
		// pri($query_param);
		$this->query = $this->connection->query($query_param);

		if(mysqli_error($this->connection)){
			$data['error'] = mysqli_error($this->connection);
			return $data;
		}

		if($escape == true){
			$query_param = $this->escape($query_param);
		}
		if(is_null($this->query) == false){
			if(isset($this->query->num_rows)){
				$this->num_rows = $this->query->num_rows;
			}
		}
		return $this;
	}

	public function multi_query($sql = null) {
		if (empty($sql)) {
			throw new Exception('Empty sql query');
		}

		$this->connection->multi_query($sql);
		while ($this->connection->next_result()) {;}
		return $this;
	}

	/**
	 * SELECT
	 *
	 * @param $columns = '*'
	 *
	 * @return $this
	 */
	public function select($columns = '*'){
		if(is_array($columns)){
			$select = null;
			foreach($columns as $column){
				$select .= $this->escape($column) . ', ';
			}
			$select = substr($select, 0, - 2);
		}else{
			$select = $this->escape($columns);
		}
		$this->query = 'SELECT ' . $select;

		return $this;
	}

	/**
	 * FROM
	 *
	 * @param $table
	 *
	 * @return $this
	 */
	public function from($table){
		if(empty($table)){
			exit('empty table name');
		}
		$this->query .= ' FROM `' . $this->escape($table) . '`';

		return $this;
	}

	/**
	 * ORDER
	 *
	 * @param $by = string
	 * @param $tyoe = string
	 *
	 * @return $this
	*/
	public function order( $by, $type = 'ASC' ){
		if(empty($by))
			exit('order by cant be empty');
		$type = ((strtoupper($type) == 'ASC' or strtoupper($type) == 'DESC') ? strtoupper($type) : 'ASC');
		$this->query .= ' ORDER BY ' . $this->escape($by) . ' ' . $type;

		return $this;
	}

	/**
	 * WHERE
	 *
	 * @param $data = array
	 * @param $return = string
	 *
	 * @return $this
	 */
	public function where($data, $return = false){
		if(is_array($data)){

			if(isset($data['operator'])){
				$operator = $data['operator'];
				unset($data['operator']);
			}else{
				$operator = 'AND';
			}
			$o = strtoupper($operator);
			$tmp = null;
			foreach($data as $k => $v){
				if(is_array($v)){
					$extra = ((isset($extra)) ? $extra + strlen($k) : strlen($k));
					$tmp .= $this->escape($k) . ' = ';
					foreach($v as $nv){
						$tmp .= ((is_numeric($nv) && intval($nv) == $nv) ? $this->escape($nv) . ' ' . $o . ' ' : "'{$this->escape($nv)}'" . ' ' . $o . ' ');
						$tmp .= $this->escape($k) . ' = ';
					}
				}else{
					$tmp .= $this->escape($k) . ' = ' . ((is_numeric($v) && intval($v) == $v) ? $v . ' ' . $o . ' ' : "'{$this->escape($v)}'" . ' ' . $o . ' ');
				}
			}
			$where = substr($tmp, 0, (isset($extra)) ? - (strlen($o) + 2 + $extra + 3) : - (strlen($o) + 2));
		}else{
			$where = $data;
		}
		$this->wherere = $where;
		if($return == false){
			$this->query .= ' WHERE ' . $where;

			return $this;
		}else{
			return $where;
		}
	}

	/**
	 * GETROWS
	 * @return $num_rows
	 */
	public function getRows(){
		return $this->num_rows;
	}

	/**
	 * FETCHSINGLE
	 *
	 * @param $col = null; name colum return
	 * @param $type = MYSQLI_ASSOC
	 *
	 * @return $result
	 */
	public function fetchSingle($col = null, $type = MYSQLI_ASSOC){
		if(is_string($this->query)){
			$this->query($this->query);
		}
		$result = $this->query->fetch_array($type);
		$this->query->free();

		return isset($result) ? ((isset($col)) ? $result[ $col ] : $result) : null;
	}

	/**
	 * FETCH
	 *
	 * @param $type = MYSQLI_ASSOC
	 *
	 * @return $data; from query
	 */
	public function fetch($type = MYSQLI_ASSOC,$more = false){
		$data = null;
		
		if($more){
			$this->query($this->query);
		}
		while($result = $this->query->fetch_array($type)){
			$data[] = $result;
		}
		$this->query->free();

		return $data;
	}

	/**
	 * INSERT
	 *
	 * @param $table ; table name
	 * @param $data ; array
	 *
	 * @return $this
	 */
	public function insert($table, array $data){
		if(is_array($data)){
			$columns = null;
			$values  = null;
			foreach($data as $k => $v){
				$columns .= $this->escape($k) . ', ';
				$values .= ((is_int($v) && intval($v) == $v && substr($v, 0,
					1) != 0) ? $this->escape($v) . ', ' : "'{$this->escape($v)}'" . ', ');
			}
			$columns = substr($columns, 0, - 2);
			$values  = substr($values, 0, - 2);
		}
		$this->query('INSERT INTO `' . $this->escape($table) . '` (' . $columns . ') VALUES(' . $values . ')', false);

		return $this;
	}

	/**
	 * DELETE
	 *
	 * @param $table ; table name
	 * @param $condition ; where
	 * @param $limit ; string
	 *
	 * @return $this
	 */
	public function delete($table, $condition = null, $limit = null){
		if(empty($table)){
			exit('empty table');
		}
		if(is_array($condition)){
			$where = ' WHERE ' . $this->where($condition, true);
		}else{
			$where = ' WHERE ' . $this->escape($condition);
		}

		$limit = ((isset($limit)) ? ' LIMIT ' . (int) $this->escape($limit) : '');
		$this->query('DELETE FROM `' . $this->escape($table) . '`' . $where . $limit);

		return $this;
	}

	/**
	 * UPDATE
	 *
	 * @param $table ; table name
	 * @param $data ; array, string
	 * @param $condition ; where
	 *
	 * @return $this
	 */
	public function update($table, $data, $condition = null){
		if(is_array($data)){
			$tmp = null;
			foreach($data as $k => $v){
				$tmp .= $this->escape($k) . ' = ' . ((is_numeric($v) && intval($v) == $v && substr($v, 0,
					1) != 0) ? $this->escape($v) . ', ' : "'{$this->escape($v)}'" . ', ');
			}

			$values = substr($tmp, 0, - 2);
		}else{
			$values = $this->escape($data);
		}

		if(isset($condition)){
			$where = ' WHERE ';
			if(is_array($condition)){
				$where .= $this->where($condition, true);
			}else{
				$where .= $this->escape($condition);
			}
		}
		$q = 'UPDATE `' . $this->escape($table) . '` SET ' . $values . ((isset($where)) ? $where : '');;
		$this->query($q);
		return $this;
	}

	/**
	 * ESCAPE
	 *
	 * @param $string
	 *
	 * @return $string
	 */
	public function escape($string){
		return $this->connection->real_escape_string($string);
	}

	/**
	 * AUTO_INCREMENT
	 *
	 * @param $table
	 *
	 * @return auto_increment
	 */
	public function auto_increment($table){
		return $this->query("
			SELECT `AUTO_INCREMENT` 
			FROM  INFORMATION_SCHEMA.TABLES 
			WHERE TABLE_SCHEMA = '" . $this->dbOptions['database'] . "' 
			AND   
			TABLE_NAME   = '$table'
			")->fetch()[0]['AUTO_INCREMENT'];
	}
}
